module BasecommSdk
  class BankAccountTransaction < Base
    ACCOUNT_TYPE = {
      xs_bat_account_type_checking: 'CHECKING',
      xs_bat_account_type_savings:  'SAVINGS'
    }
    TYPE = {
      xs_bat_type_credit:   'CREDIT',
      xs_bat_type_debit:    'DEBIT',
      xs_bat_type_reversal: 'REVERSAL',
      xs_bat_type_cancel:   'CANCEL'
    }
    STATUS = {
      xs_bat_status_created:   'CREATED',
      xs_bat_status_initiated: 'INITIATED',
      xs_bat_status_settled:   'SETTLED',
      xs_bat_status_returned:  'RETURNED',
      xs_bat_status_canceled:  'CANCELED',
      xs_bat_status_failed:    'FAILED',
      xs_bat_status_rejected:  'REJECTED'
    }
    METHOD = {
      xs_bat_method_ccd: 'CCD',
      xs_bat_method_ppd: 'PPD',
      xs_bat_method_tel: 'TEL',
      xs_bat_method_web: 'WEB'
    }

    attr_accessor :id, :type, :status, :method,
      :account_type, :routing_number, :account_number, :account_name,
      :return_code, :trace, :merchant_transaction_id, :bank_account_token,
      :micr_data, :amount, :effective_date, :settlement_date, :return_date,
      :recurring_indicator, :creation_date

    attr_accessor :custom_field1, :custom_field2, :custom_field3,
      :custom_field4, :custom_field5, :custom_field6, :custom_field7,
      :custom_field8, :custom_field9, :custom_field10

    def self.build_from_json(json)
      BankAccountTransaction.new(JSON.parse(json).recursively_symbolize_keys!)
    end
    singleton_class.send(:alias_method, :from_json, :build_from_json)

    def json_prefix
      :bank_account_transaction
    end

    def correction?
      return false if return_code.nil?
      status_name == STATUS[:xs_bat_status_returned] && !return_code.match(/^C/).nil?
    end

    def returned?
      return false if correction?
      status_name == STATUS[:xs_bat_status_returned] && !return_code.match(/^R/).nil?
    end

    def settled?
      status_name == STATUS[:xs_bat_status_settled] && !settlement_date.nil?
    end

    def failed?
      status_name == STATUS[:xs_bat_status_failed] ? true : false
    end

    def status_date
      the_date = case status_name
        when STATUS[:xs_bat_status_created]
          creation_date
        when STATUS[:xs_bat_status_returned]
          return_date
        when STATUS[:xs_bat_status_settled]
          settlement_date
        when STATUS[:xs_bat_status_initiated]
          effective_date
        else
          Time.now.strftime('%m/%d/%Y %H:%M:%S')
        end

      DateTime.strptime(the_date, '%m/%d/%Y %H:%M:%S')
    end

    def account_name=(name)
      @account_name = name.to_s.slice(0, 32).strip
    end
  end
end
