# rubocop:disable Style/TrivialAccessors
module BasecommSdk
  class Base
    def self.attr_accessor(*vars)
      @attributes ||= []
      @attributes.concat(vars)
      super(*vars)
    end

    def self.attributes
      @attributes
    end

    # Accepts prefixed attribute keys as well as non-prefixed
    def initialize(options = {})
      @messages = []
      instance_hash = options.fetch(json_prefix, options)
      attributes.each do |atr|
        instance_variable_set(
          "@#{atr}",
          instance_hash.fetch(
            "#{json_prefix}_#{atr}".to_sym,
            instance_hash.fetch(
              "#{atr}".to_sym,
              nil
            )
          )
        )
      end
      # Populate messages with exceptions
      if options[:exception]
        options[:exception].each_pair do |k, v|
          messages << {k => v}
        end
      end
    end

    def messages
      @messages
    end

    def attributes
      self.class.attributes
    end

    def qualified_attributes
      array = []
      attributes.each do |atr|
        array << qualified_key(atr)
      end
      array
    end

    def to_json(_options = {})
      hash = {}
      instance_variables.each do |iv|
        var_name = iv.to_s[1..-1]
        next if var_name.match(/messages$/)
        var = instance_variable_get(iv)
        hash[qualified_key(var_name)] = format(var) unless var.nil?
      end
      hash.to_json
    end

    def self.json_prefix
      new.json_prefix
    end

    def qualified_key(key)
      "#{json_prefix}_#{key}".to_sym
    end

    def format(instance_var)
      case instance_var.class.name
      when 'Date', 'Time', 'DateTime'
        instance_var.strftime('%m/%d/%Y %H:%M:%S')
      else
        instance_var
      end
    end

    def status_name
      status["#{qualified_key('status')}_name".to_sym]
    end

    def status_description
      status["#{qualified_key('status')}_description".to_sym]
    end
  end
end
# rubocop:enable Style/TrivialAccessors
